@@ -5,13 +5,15 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  */
 
-#ifndef ORCUS_ORCUS_ODS_HPP
-#define ORCUS_ORCUS_ODS_HPP
+#ifndef INCLUDED_ORCUS_ORCUS_ODS_HPP
+#define INCLUDED_ORCUS_ORCUS_ODS_HPP
 
 #include "orcus/spreadsheet/import_interface.hpp"
 #include "orcus/env.hpp"
 #include "interface.hpp"
 
+#include <memory>
+
 namespace orcus {
 
 namespace spreadsheet { namespace iface { class import_factory; }}
@@ -31,21 +33,24 @@ public:
 
     static bool detect(const unsigned char* blob, size_t size);
 
-    virtual void read_file(const std::string& filepath);
+    virtual void read_file(std::string_view filepath) override;
+    virtual void read_file(std::u16string_view filepath) override;
 
-    virtual void read_stream(const char* content, size_t len);
+    virtual void read_stream(std::string_view stream) override;
 
-    virtual const char* get_name() const;
+    virtual std::string_view get_name() const override;
 
 private:
     static void list_content(const zip_archive& archive);
+    void read_styles(const zip_archive& archive);
     void read_content(const zip_archive& archive);
     void read_content_xml(const unsigned char* p, size_t size);
 
     void read_file_impl(zip_archive_stream* stream);
 
 private:
-    orcus_ods_impl* mp_impl;
+    struct impl;
+    std::unique_ptr<impl> mp_impl;
 };
 
 }
