@@ -5,10 +5,8 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  */
 
-#include <orcus/global.hpp>
 #include <orcus/measurement.hpp>
 #include <orcus/spreadsheet/types.hpp>
-#include "pstring.hpp"
 
 #include <cstdlib>
 #include <cassert>
@@ -16,7 +14,6 @@
 #include <sstream>
 #include <cmath>
 
-using namespace std;
 using namespace orcus;
 
 void test_date_time_conversion()
@@ -36,11 +33,11 @@ void test_date_time_conversion()
 
     for (size_t i = 0, n = sizeof(tests)/sizeof(tests[0]); i < n; ++i)
     {
-        pstring str(tests[i].str);
-        date_time_t ret = to_date_time(str);
-        cout << "original: " << str << endl;
-        cout << "converted: year=" << ret.year << ", month=" << ret.month << ", day="
-            << ret.day << ", hour=" << ret.hour << ", minute=" << ret.minute << ", second=" << ret.second << endl;
+        std::string_view str(tests[i].str);
+        date_time_t ret = date_time_t::from_chars(str);
+        std::cout << "original: " << str << std::endl;
+        std::cout << "converted: year=" << ret.year << ", month=" << ret.month << ", day="
+            << ret.day << ", hour=" << ret.hour << ", minute=" << ret.minute << ", second=" << ret.second << std::endl;
         assert(ret.year == tests[i].year);
         assert(ret.month == tests[i].month);
         assert(ret.day == tests[i].day);
@@ -50,7 +47,7 @@ void test_date_time_conversion()
     }
 }
 
-string to_string(length_unit_t unit)
+std::string to_string(length_unit_t unit)
 {
     switch (unit)
     {
@@ -97,9 +94,9 @@ void test_measurement_conversion()
     for (size_t i = 0, n = sizeof(tests)/sizeof(tests[0]); i < n; ++i)
     {
         length_t ret = to_length(tests[i].str);
-        cout << "original: '" << tests[i].str << "', converted: " << ret.value
+        std::cout << "original: '" << tests[i].str << "', converted: " << ret.value
             << " (" << to_string(ret.unit) << "), expected: "
-            << tests[i].expected << " (" << to_string(tests[i].unit) << ")" << endl;
+            << tests[i].expected << " (" << to_string(tests[i].unit) << ")" << std::endl;
 
         // Check for double-precision equality without the rounding error.
         double factor = 1.0;
@@ -160,8 +157,8 @@ void test_string2number_conversion()
         const char* p = tests[i].str;
         const char* p_parse_ended = nullptr;
         double converted = to_double(p, &p_parse_ended);
-        cout << "original: '" << tests[i].str << "', converted: " << converted
-            << ", expected: " << tests[i].expected << endl;
+        std::cout << "original: '" << tests[i].str << "', converted: " << converted
+            << ", expected: " << tests[i].expected << std::endl;
 
         // Check for double-precision equality without the rounding error.
         double factor = 1.0;
@@ -197,8 +194,8 @@ void test_string2long_conversion()
         const char* p = tests[i].str;
         const char* p_parse_ended = nullptr;
         long converted = to_long(p, &p_parse_ended);
-        cout << "original: '" << tests[i].str << "', converted: " << converted
-            << ", expected: " << tests[i].expected << endl;
+        std::cout << "original: '" << tests[i].str << "', converted: " << converted
+            << ", expected: " << tests[i].expected << std::endl;
 
         assert(converted == tests[i].expected);
 
@@ -208,40 +205,6 @@ void test_string2long_conversion()
     }
 }
 
-void test_pstring_trim()
-{
-    // test for trimming.
-    string s1("test"), s2("  test"), s3("   test  "), s4("test   ");
-    pstring ps1(s1.c_str()), ps2(s2.c_str()), ps3(s3.c_str()), ps4(s4.c_str());
-    assert(ps1 != ps2);
-    assert(ps1 != ps3);
-    assert(ps2 != ps3);
-    assert(ps1 != ps4);
-
-    pstring trimmed = ps1.trim();
-    assert(ps1 == trimmed); // nothing to trim.
-    assert(ps1 == ps2.trim());
-    assert(ps1 == ps3.trim());
-    assert(ps1 == ps4.trim());
-    assert(ps1.size() == ps2.trim().size());
-    assert(ps1.size() == ps3.trim().size());
-}
-
-void test_pstring_equality()
-{
-    // Two pstrings that point to the same buffer but different sizes should
-    // not be equal.
-    string s1("some string");
-    pstring p1(s1.data(), 3), p2(s1.data(), 6);
-    assert(p1 != p2);
-
-    // Two pstrings that point to different buffers but with same content
-    // should be equal.
-    string s2("some string");
-    pstring p3(s2.data(), 6);
-    assert(p2 == p3);
-}
-
 void test_spreadsheet_types()
 {
     std::vector<spreadsheet::error_value_t> values =
@@ -273,8 +236,6 @@ int main()
     test_measurement_conversion_2();
     test_string2number_conversion();
     test_string2long_conversion();
-    test_pstring_trim();
-    test_pstring_equality();
     test_spreadsheet_types();
 
     return EXIT_SUCCESS;
