@@ -11,8 +11,8 @@
 namespace orcus {
 
 gnumeric_content_xml_handler::gnumeric_content_xml_handler(
-    session_context& session_cxt, const tokens& tokens, spreadsheet::iface::import_factory* factory) :
-    xml_stream_handler(new gnumeric_content_xml_context(session_cxt, tokens, factory))
+    session_context& session_cxt, const tokens& t, spreadsheet::iface::import_factory* factory) :
+    xml_stream_handler(session_cxt, t, std::make_unique<gnumeric_content_xml_context>(session_cxt, t, factory))
 {
 }
 
