@@ -14,21 +14,16 @@ ods_dde_links_context::ods_dde_links_context(session_context& session_cxt, const
 
 ods_dde_links_context::~ods_dde_links_context() {}
 
-bool ods_dde_links_context::can_handle_element(xmlns_id_t ns, xml_token_t name) const
-{
-    return true;
-}
-
-xml_context_base* ods_dde_links_context::create_child_context(xmlns_id_t ns, xml_token_t name)
+xml_context_base* ods_dde_links_context::create_child_context(xmlns_id_t /*ns*/, xml_token_t /*name*/)
 {
     return nullptr;
 }
 
-void ods_dde_links_context::end_child_context(xmlns_id_t ns, xml_token_t name, xml_context_base *child)
+void ods_dde_links_context::end_child_context(xmlns_id_t /*ns*/, xml_token_t /*name*/, xml_context_base* /*child*/)
 {
 }
 
-void ods_dde_links_context::start_element(xmlns_id_t ns, xml_token_t name, const::std::vector<xml_token_attr_t> &attrs)
+void ods_dde_links_context::start_element(xmlns_id_t ns, xml_token_t name, const::std::vector<xml_token_attr_t>& /*attrs*/)
 {
     xml_token_pair_t parent = push_stack(ns, name);
     (void)parent;
@@ -41,7 +36,7 @@ bool ods_dde_links_context::end_element(xmlns_id_t ns, xml_token_t name)
     return pop_stack(ns, name);
 }
 
-void ods_dde_links_context::characters(const pstring &str, bool transient)
+void ods_dde_links_context::characters(std::string_view /*str*/, bool /*transient*/)
 {
 }
 
