@@ -28,15 +28,13 @@
 #include <boost/iostreams/filtering_stream.hpp>
 #include <boost/iostreams/filter/gzip.hpp>
 
-using namespace std;
-
 namespace orcus {
 
 namespace {
 
-bool decompress_gzip(const char* buffer, size_t size, string& decompressed)
+bool decompress_gzip(const char* buffer, size_t size, std::string& decompressed)
 {
-    string buf;
+    std::string buf;
 
     try
     {
@@ -46,7 +44,7 @@ bool decompress_gzip(const char* buffer, size_t size, string& decompressed)
         boost::iostreams::write(os, buffer, size);
         os.flush();
     }
-    catch (const exception&)
+    catch (const std::exception&)
     {
         return false;
     }
@@ -93,7 +91,7 @@ bool orcus_gnumeric::detect(const unsigned char* buffer, size_t size)
 {
     // Detect gnumeric format that's already in memory.
 
-    string decompressed;
+    std::string decompressed;
     if (!decompress_gzip(reinterpret_cast<const char*>(buffer), size, decompressed))
         return false;
 
@@ -122,19 +120,25 @@ bool orcus_gnumeric::detect(const unsigned char* buffer, size_t size)
     return false;
 }
 
-void orcus_gnumeric::read_file(const string& filepath)
+void orcus_gnumeric::read_file(std::string_view filepath)
 {
 #if ORCUS_DEBUG_GNUMERIC
     cout << "reading " << filepath << endl;
 #endif
 
-    file_content content(filepath.data());
+    file_content content(filepath);
     if (content.empty())
         return;
 
     read_stream(content.str());
 }
 
+void orcus_gnumeric::read_file(std::u16string_view filepath)
+{
+    orcus::file_content fc(filepath);
+    read_stream(fc.str());
+}
+
 void orcus_gnumeric::read_stream(std::string_view stream)
 {
     if (stream.empty())
@@ -144,6 +148,12 @@ void orcus_gnumeric::read_stream(std::string_view stream)
     if (!decompress_gzip(stream.data(), stream.size(), file_content))
         return;
 
+    if (auto* gs = mp_impl->mp_factory->get_global_settings(); gs)
+    {
+        gs->set_origin_date(1899, 12, 30);
+        gs->set_default_formula_grammar(spreadsheet::formula_grammar_t::gnumeric);
+    }
+
     mp_impl->read_content_xml(file_content, get_config());
     mp_impl->mp_factory->finalize();
 }
