@@ -24,8 +24,6 @@
 #include <locale>
 #include <codecvt>
 
-using namespace std;
-
 namespace orcus {
 
 struct orcus_xls_xml::impl
@@ -61,7 +59,7 @@ struct orcus_xls_xml::impl
         }
         catch (const parse_error& e)
         {
-            std::cerr << create_parse_error_output(pstring(content, len), e.offset()) << std::endl;
+            std::cerr << create_parse_error_output(std::string_view(content, len), e.offset()) << std::endl;
             std::cerr << e.what() << std::endl;
             return;
         }
@@ -77,14 +75,17 @@ orcus_xls_xml::orcus_xls_xml(spreadsheet::iface::import_factory* factory) :
     mp_impl->m_ns_repo.add_predefined_values(NS_xls_xml_all);
 }
 
-orcus_xls_xml::~orcus_xls_xml() {}
+orcus_xls_xml::~orcus_xls_xml() = default;
 
 bool orcus_xls_xml::detect(const unsigned char* buffer, size_t size)
 {
+    memory_content mem_content({reinterpret_cast<const char*>(buffer), size});
+    mem_content.convert_to_utf8();
+
     config opt(format_t::xls_xml);
     xmlns_repository ns_repo;
     ns_repo.add_predefined_values(NS_xls_xml_all);
-    xml_stream_parser parser(opt, ns_repo, xls_xml_tokens, reinterpret_cast<const char*>(buffer), size);
+    xml_stream_parser parser(opt, ns_repo, xls_xml_tokens, mem_content.data(), mem_content.size());
 
     session_context cxt;
     xls_xml_detection_handler handler(cxt, xls_xml_tokens);
@@ -102,7 +103,7 @@ bool orcus_xls_xml::detect(const unsigned char* buffer, size_t size)
     return false;
 }
 
-void orcus_xls_xml::read_file(const string& filepath)
+void orcus_xls_xml::read_file(std::string_view filepath)
 {
     file_content content(filepath.data());
     if (content.empty())
@@ -112,6 +113,12 @@ void orcus_xls_xml::read_file(const string& filepath)
     mp_impl->read_stream(content.data(), content.size(), get_config());
 }
 
+void orcus_xls_xml::read_file(std::u16string_view filepath)
+{
+    orcus::file_content fc(filepath);
+    read_stream(fc.str());
+}
+
 void orcus_xls_xml::read_stream(std::string_view stream)
 {
     memory_content mem_content(stream);
