@@ -7,16 +7,17 @@
 
 #include "orcus/orcus_xlsx.hpp"
 
-#include "orcus/xml_namespace.hpp"
-#include "orcus/global.hpp"
-#include "orcus/spreadsheet/import_interface.hpp"
-#include "orcus/exception.hpp"
-#include "orcus/config.hpp"
-#include "orcus/measurement.hpp"
+#include <orcus/xml_namespace.hpp>
+#include <orcus/spreadsheet/import_interface.hpp>
+#include <orcus/exception.hpp>
+#include <orcus/config.hpp>
+#include <orcus/measurement.hpp>
+#include <orcus/stream.hpp>
 
 #include "xlsx_types.hpp"
 #include "xlsx_handler.hpp"
-#include "xlsx_context.hpp"
+#include "xlsx_shared_strings_context.hpp"
+#include "xlsx_styles_context.hpp"
 #include "xlsx_workbook_context.hpp"
 #include "xlsx_revision_context.hpp"
 #include "ooxml_tokens.hpp"
@@ -38,7 +39,7 @@
 #include <cstring>
 #include <sstream>
 
-using namespace std;
+namespace ss = orcus::spreadsheet;
 
 namespace orcus {
 
@@ -124,7 +125,7 @@ struct orcus_xlsx::impl
     opc_reader m_opc_reader;
 
     impl(spreadsheet::iface::import_factory* factory, orcus_xlsx& parent) :
-        m_cxt(new xlsx_session_data),
+        m_cxt(std::make_unique<xlsx_session_data>()),
         mp_factory(factory),
         m_opc_handler(parent),
         m_opc_reader(parent.get_config(), m_ns_repo, m_cxt, m_opc_handler) {}
@@ -155,55 +156,52 @@ bool orcus_xlsx::detect(const unsigned char* blob, size_t size)
 {
     zip_archive_stream_blob stream(blob, size);
     zip_archive archive(&stream);
+
     try
     {
         archive.load();
+
+        // Find and parse [Content_Types].xml which is required for OPC package.
+        std::vector<unsigned char> buf = archive.read_file_entry("[Content_Types].xml");
+
+        if (buf.empty())
+            return false;
+
+        config opt(format_t::xlsx);
+        xmlns_repository ns_repo;
+        ns_repo.add_predefined_values(NS_opc_all);
+        session_context session_cxt;
+        xml_stream_parser parser(
+            opt, ns_repo, opc_tokens, reinterpret_cast<const char*>(&buf[0]), buf.size());
+
+        xml_stream_handler handler(
+            session_cxt, opc_tokens,
+            std::make_unique<opc_content_types_context>(session_cxt, opc_tokens));
+        parser.set_handler(&handler);
+        parser.parse();
+
+        auto& context = static_cast<opc_content_types_context&>(handler.get_root_context());
+
+        std::vector<xml_part_t> parts;
+        context.pop_parts(parts);
+
+        if (parts.empty())
+            return false;
+
+        // See if we can find the workbook stream.
+        xml_part_t workbook_part("/xl/workbook.xml", CT_ooxml_xlsx_sheet_main);
+        return std::find(parts.begin(), parts.end(), workbook_part) != parts.end();
     }
-    catch (const zip_error&)
+    catch (const std::exception&)
     {
-        // Not a valid zip archive.
         return false;
     }
-
-    // Find and parse [Content_Types].xml which is required for OPC package.
-    vector<unsigned char> buf;
-    if (!archive.read_file_entry("[Content_Types].xml", buf))
-        // Failed to read the contnet types entry.
-        return false;
-
-    if (buf.empty())
-        return false;
-
-    config opt(format_t::xlsx);
-    xmlns_repository ns_repo;
-    ns_repo.add_predefined_values(NS_opc_all);
-    session_context session_cxt;
-    xml_stream_parser parser(
-        opt, ns_repo, opc_tokens, reinterpret_cast<const char*>(&buf[0]), buf.size());
-
-    xml_simple_stream_handler handler(
-        session_cxt, opc_tokens,
-        std::make_unique<opc_content_types_context>(session_cxt, opc_tokens));
-    parser.set_handler(&handler);
-    parser.parse();
-
-    opc_content_types_context& context =
-        static_cast<opc_content_types_context&>(handler.get_context());
-
-    std::vector<xml_part_t> parts;
-    context.pop_parts(parts);
-
-    if (parts.empty())
-        return false;
-
-    // See if we can find the workbook stream.
-    xml_part_t workbook_part("/xl/workbook.xml", CT_ooxml_xlsx_sheet_main);
-    return std::find(parts.begin(), parts.end(), workbook_part) != parts.end();
 }
 
-void orcus_xlsx::read_file(const string& filepath)
+void orcus_xlsx::read_file(std::string_view filepath)
 {
-    std::unique_ptr<zip_archive_stream> stream(new zip_archive_stream_fd(filepath.c_str()));
+    std::unique_ptr<zip_archive_stream> stream(
+        new zip_archive_stream_fd(std::string{filepath}.c_str()));
     mp_impl->m_opc_reader.read_file(std::move(stream));
 
     // Formulas need to be inserted to the document after the shared string
@@ -214,6 +212,12 @@ void orcus_xlsx::read_file(const string& filepath)
     mp_impl->mp_factory->finalize();
 }
 
+void orcus_xlsx::read_file(std::u16string_view filepath)
+{
+    orcus::file_content fc(filepath);
+    read_stream(fc.str());
+}
+
 void orcus_xlsx::read_stream(std::string_view stream)
 {
     std::unique_ptr<zip_archive_stream> blob(
@@ -256,7 +260,7 @@ void orcus_xlsx::set_formulas_to_doc()
         }
     };
 
-    xlsx_session_data& sdata = static_cast<xlsx_session_data&>(*mp_impl->m_cxt.mp_data);
+    auto& sdata = mp_impl->m_cxt.get_data<xlsx_session_data>();
 
     // Insert shared formulas first.
     for (auto& p : sdata.m_shared_formulas)
@@ -338,25 +342,25 @@ size_t get_schema_rank(const schema_t sch)
     }
 
     auto it = rank_map.find(sch);
-    return it == rank_map.end() ? numeric_limits<size_t>::max() : it->second;
+    return it == rank_map.end() ? std::numeric_limits<size_t>::max() : it->second;
 }
 
 }
 
-void orcus_xlsx::read_workbook(const string& dir_path, const string& file_name)
+void orcus_xlsx::read_workbook(const std::string& dir_path, const std::string& file_name)
 {
-    string filepath = resolve_file_path(dir_path, file_name);
+    std::string filepath = resolve_file_path(dir_path, file_name);
     if (get_config().debug)
-        cout << "read_workbook: file path = " << filepath << endl;
+        std::cout << "read_workbook: file path = " << filepath << std::endl;
 
-    vector<unsigned char> buffer;
+    std::vector<unsigned char> buffer;
     if (!mp_impl->m_opc_reader.open_zip_stream(filepath, buffer))
         return;
 
     if (buffer.empty())
         return;
 
-    auto handler = std::make_unique<xml_simple_stream_handler>(
+    auto handler = std::make_unique<xml_stream_handler>(
         mp_impl->m_cxt, ooxml_tokens,
         std::make_unique<xlsx_workbook_context>(mp_impl->m_cxt, ooxml_tokens, *mp_impl->mp_factory));
 
@@ -367,32 +371,23 @@ void orcus_xlsx::read_workbook(const string& dir_path, const string& file_name)
     parser.parse();
 
     // Get sheet info from the context instance.
-    xlsx_workbook_context& context =
-        static_cast<xlsx_workbook_context&>(handler->get_context());
+    auto& context = static_cast<xlsx_workbook_context&>(handler->get_root_context());
     opc_rel_extras_t workbook_data;
     context.pop_workbook_info(workbook_data);
     if (get_config().debug)
     {
-        for_each(workbook_data.data.begin(), workbook_data.data.end(),
-            [](const opc_rel_extras_t::map_type::value_type& v)
+        for (const auto& [rid, extra] : workbook_data.data)
+        {
+            if (auto* info = dynamic_cast<const xlsx_rel_sheet_info*>(extra.get()); info)
             {
-                const xlsx_rel_sheet_info* info =
-                    dynamic_cast<const xlsx_rel_sheet_info*>(v.second.get());
-
-                if (info)
-                {
-                    cout << "relationship id: " << v.first << "; sheet name: " << info->name << "; sheet id: " << info->id << endl;
-                }
-
-                const xlsx_rel_pivot_cache_info* info_pc =
-                    dynamic_cast<const xlsx_rel_pivot_cache_info*>(v.second.get());
-
-                if (info_pc)
-                {
-                    cout << "relationship id: " << v.first << "; pivot cache id: " << info_pc->id << endl;
-                }
+                std::cout << "relationship id: " << rid << "; sheet name: " << info->name << "; sheet id: " << info->id << std::endl;
             }
-        );
+
+            if (auto* info_pc = dynamic_cast<const xlsx_rel_pivot_cache_info*>(extra.get()); info_pc)
+            {
+                std::cout << "relationship id: " << rid << "; pivot cache id: " << info_pc->id << std::endl;
+            }
+        }
     }
 
     handler.reset();
@@ -407,13 +402,13 @@ void orcus_xlsx::read_workbook(const string& dir_path, const string& file_name)
             if (rank_left != rank_right)
                 return rank_left < rank_right;
 
-            pstring rid1 = left.rid, rid2 = right.rid;
+            std::string_view rid1 = left.rid, rid2 = right.rid;
 
             if (rid1.size() > 1 && rid2.size() > 1)
             {
                 // numerical comparison of relation ID's.
-                rid1 = pstring(&rid1[1], rid1.size()-1); // remove the 'r' prefix.
-                rid2 = pstring(&rid2[1], rid2.size()-1); // remove the 'r' prefix.
+                rid1 = std::string_view(rid1.data()+1, rid1.size()-1); // remove the 'r' prefix.
+                rid2 = std::string_view(rid2.data()+1, rid2.size()-1); // remove the 'r' prefix.
                 return to_long(rid1) < to_long(rid2);
             }
 
@@ -424,20 +419,21 @@ void orcus_xlsx::read_workbook(const string& dir_path, const string& file_name)
     mp_impl->m_opc_reader.check_relation_part(file_name, &workbook_data, &sort_func);
 }
 
-void orcus_xlsx::read_sheet(const string& dir_path, const string& file_name, xlsx_rel_sheet_info* data)
+void orcus_xlsx::read_sheet(
+    const std::string& dir_path, const std::string& file_name, xlsx_rel_sheet_info* data)
 {
     if (!data || !data->id)
         // Sheet ID must not be 0.
         return;
 
-    string filepath = resolve_file_path(dir_path, file_name);
+    std::string filepath = resolve_file_path(dir_path, file_name);
     if (get_config().debug)
     {
-        cout << "---" << endl;
-        cout << "read_sheet: file path = " << filepath << endl;
+        std::cout << "---" << std::endl;
+        std::cout << "read_sheet: file path = " << filepath << std::endl;
     }
 
-    vector<unsigned char> buffer;
+    std::vector<unsigned char> buffer;
     if (!mp_impl->m_opc_reader.open_zip_stream(filepath, buffer))
         return;
 
@@ -446,8 +442,8 @@ void orcus_xlsx::read_sheet(const string& dir_path, const string& file_name, xls
 
     if (get_config().debug)
     {
-        cout << "relationship sheet data: " << endl;
-        cout << "  sheet name: " << data->name << "  sheet ID: " << data->id << endl;
+        std::cout << "relationship sheet data: " << std::endl;
+        std::cout << "  sheet name: " << data->name << "  sheet ID: " << data->id << std::endl;
     }
 
     spreadsheet::iface::import_sheet* sheet = mp_impl->mp_factory->get_sheet(data->name);
@@ -480,16 +476,16 @@ void orcus_xlsx::read_sheet(const string& dir_path, const string& file_name, xls
     mp_impl->m_opc_reader.check_relation_part(file_name, &table_info);
 }
 
-void orcus_xlsx::read_shared_strings(const string& dir_path, const string& file_name)
+void orcus_xlsx::read_shared_strings(const std::string& dir_path, const std::string& file_name)
 {
-    string filepath = resolve_file_path(dir_path, file_name);
+    std::string filepath = resolve_file_path(dir_path, file_name);
     if (get_config().debug)
     {
-        cout << "---" << endl;
-        cout << "read_shared_strings: file path = " << filepath << endl;
+        std::cout << "---" << std::endl;
+        std::cout << "read_shared_strings: file path = " << filepath << std::endl;
     }
 
-    vector<unsigned char> buffer;
+    std::vector<unsigned char> buffer;
     if (!mp_impl->m_opc_reader.open_zip_stream(filepath, buffer))
         return;
 
@@ -500,7 +496,7 @@ void orcus_xlsx::read_shared_strings(const string& dir_path, const string& file_
         get_config(), mp_impl->m_ns_repo, ooxml_tokens,
         reinterpret_cast<const char*>(&buffer[0]), buffer.size());
 
-    auto handler = std::make_unique<xml_simple_stream_handler>(
+    auto handler = std::make_unique<xml_stream_handler>(
         mp_impl->m_cxt, ooxml_tokens,
         std::make_unique<xlsx_shared_strings_context>(
             mp_impl->m_cxt, ooxml_tokens, mp_impl->mp_factory->get_shared_strings()));
@@ -509,13 +505,13 @@ void orcus_xlsx::read_shared_strings(const string& dir_path, const string& file_
     parser.parse();
 }
 
-void orcus_xlsx::read_styles(const string& dir_path, const string& file_name)
+void orcus_xlsx::read_styles(const std::string& dir_path, const std::string& file_name)
 {
-    string filepath = resolve_file_path(dir_path, file_name);
+    std::string filepath = resolve_file_path(dir_path, file_name);
     if (get_config().debug)
     {
-        cout << "---" << endl;
-        cout << "read_styles: file path = " << filepath << endl;
+        std::cout << "---" << std::endl;
+        std::cout << "read_styles: file path = " << filepath << std::endl;
     }
 
     spreadsheet::iface::import_styles* styles = mp_impl->mp_factory->get_styles();
@@ -523,7 +519,7 @@ void orcus_xlsx::read_styles(const string& dir_path, const string& file_name)
         // Client code doesn't support styles.
         return;
 
-    vector<unsigned char> buffer;
+    std::vector<unsigned char> buffer;
     if (!mp_impl->m_opc_reader.open_zip_stream(filepath, buffer))
         return;
 
@@ -534,7 +530,7 @@ void orcus_xlsx::read_styles(const string& dir_path, const string& file_name)
         get_config(), mp_impl->m_ns_repo, ooxml_tokens,
         reinterpret_cast<const char*>(&buffer[0]), buffer.size());
 
-    auto handler = std::make_unique<xml_simple_stream_handler>(
+    auto handler = std::make_unique<xml_stream_handler>(
         mp_impl->m_cxt, ooxml_tokens,
         std::make_unique<xlsx_styles_context>(
             mp_impl->m_cxt, ooxml_tokens, mp_impl->mp_factory->get_styles()));
@@ -548,7 +544,7 @@ void orcus_xlsx::read_table(const std::string& dir_path, const std::string& file
     if (!data || !data->sheet_interface)
         return;
 
-    spreadsheet::iface::import_table* table = data->sheet_interface->get_table();
+    spreadsheet::iface::import_table* table = data->sheet_interface->start_table();
     if (!table)
         // Client code doesn't support tables. No point going further.
         return;
@@ -560,17 +556,17 @@ void orcus_xlsx::read_table(const std::string& dir_path, const std::string& file
         // This client doesn't support reference resolver, but is required.
         return;
 
-    string filepath = resolve_file_path(dir_path, file_name);
+    std::string filepath = resolve_file_path(dir_path, file_name);
     if (get_config().debug)
     {
-        cout << "---" << endl;
-        cout << "read_table: file path = " << filepath << endl;
+        std::cout << "---" << std::endl;
+        std::cout << "read_table: file path = " << filepath << std::endl;
     }
 
-    vector<unsigned char> buffer;
+    std::vector<unsigned char> buffer;
     if (!mp_impl->m_opc_reader.open_zip_stream(filepath, buffer))
     {
-        cerr << "failed to open zip stream: " << filepath << endl;
+        std::cerr << "failed to open zip stream: " << filepath << std::endl;
         return;
     }
 
@@ -597,24 +593,24 @@ void orcus_xlsx::read_pivot_cache_def(
     {
         if (get_config().debug)
         {
-            cout << "---" << endl;
-            cout << "required pivot cache relation info was not present." << endl;
+            std::cout << "---" << std::endl;
+            std::cout << "required pivot cache relation info was not present." << std::endl;
         }
         return;
     }
 
-    string filepath = resolve_file_path(dir_path, file_name);
+    std::string filepath = resolve_file_path(dir_path, file_name);
     if (get_config().debug)
     {
-        cout << "---" << endl;
-        cout << "read_pivot_cache_def: file path = " << filepath
-            << "; cache id = " << data->id << endl;
+        std::cout << "---" << std::endl;
+        std::cout << "read_pivot_cache_def: file path = " << filepath
+            << "; cache id = " << data->id << std::endl;
     }
 
-    vector<unsigned char> buffer;
+    std::vector<unsigned char> buffer;
     if (!mp_impl->m_opc_reader.open_zip_stream(filepath, buffer))
     {
-        cerr << "failed to open zip stream: " << filepath << endl;
+        std::cerr << "failed to open zip stream: " << filepath << std::endl;
         return;
     }
 
@@ -651,23 +647,23 @@ void orcus_xlsx::read_pivot_cache_rec(
     {
         if (get_config().debug)
         {
-            cout << "---" << endl;
-            cout << "required pivot cache record relation info was not present." << endl;
+            std::cout << "---" << std::endl;
+            std::cout << "required pivot cache record relation info was not present." << std::endl;
         }
         return;
     }
 
-    string filepath = resolve_file_path(dir_path, file_name);
+    std::string filepath = resolve_file_path(dir_path, file_name);
     if (get_config().debug)
     {
-        cout << "---" << endl;
-        cout << "read_pivot_cache_rec: file path = " << filepath << "; cache id = " << data->id << endl;
+        std::cout << "---" << std::endl;
+        std::cout << "read_pivot_cache_rec: file path = " << filepath << "; cache id = " << data->id << std::endl;
     }
 
-    vector<unsigned char> buffer;
+    std::vector<unsigned char> buffer;
     if (!mp_impl->m_opc_reader.open_zip_stream(filepath, buffer))
     {
-        cerr << "failed to open zip stream: " << filepath << endl;
+        std::cerr << "failed to open zip stream: " << filepath << std::endl;
         return;
     }
 
@@ -694,24 +690,32 @@ void orcus_xlsx::read_pivot_cache_rec(
 
 void orcus_xlsx::read_pivot_table(const std::string& dir_path, const std::string& file_name)
 {
-    string filepath = resolve_file_path(dir_path, file_name);
+    std::string filepath = resolve_file_path(dir_path, file_name);
     if (get_config().debug)
     {
-        cout << "---" << endl;
-        cout << "read_pivot_table: file path = " << filepath << endl;
+        std::cout << "---" << std::endl;
+        std::cout << "read_pivot_table: file path = " << filepath << std::endl;
     }
 
-    vector<unsigned char> buffer;
+    std::vector<unsigned char> buffer;
     if (!mp_impl->m_opc_reader.open_zip_stream(filepath, buffer))
     {
-        cerr << "failed to open zip stream: " << filepath << endl;
+        std::cerr << "failed to open zip stream: " << filepath << std::endl;
         return;
     }
 
     if (buffer.empty())
         return;
 
-    auto handler = std::make_unique<xlsx_pivot_table_xml_handler>(mp_impl->m_cxt, ooxml_tokens);
+    auto* xpt = mp_impl->mp_factory->create_pivot_table_definition();
+    if (!xpt)
+        return;
+
+    auto* resolver = mp_impl->mp_factory->get_reference_resolver(ss::formula_ref_context_t::global);
+    if (!resolver)
+        throw general_error("orcus_xlsx::read_pivot_table: reference resolver interface is not available.");
+
+    auto handler = std::make_unique<xlsx_pivot_table_xml_handler>(mp_impl->m_cxt, ooxml_tokens, *xpt, *resolver);
 
     xml_stream_parser parser(
         get_config(), mp_impl->m_ns_repo, ooxml_tokens,
@@ -725,17 +729,17 @@ void orcus_xlsx::read_pivot_table(const std::string& dir_path, const std::string
 
 void orcus_xlsx::read_rev_headers(const std::string& dir_path, const std::string& file_name)
 {
-    string filepath = resolve_file_path(dir_path, file_name);
+    std::string filepath = resolve_file_path(dir_path, file_name);
     if (get_config().debug)
     {
-        cout << "---" << endl;
-        cout << "read_rev_headers: file path = " << filepath << endl;
+        std::cout << "---" << std::endl;
+        std::cout << "read_rev_headers: file path = " << filepath << std::endl;
     }
 
-    vector<unsigned char> buffer;
+    std::vector<unsigned char> buffer;
     if (!mp_impl->m_opc_reader.open_zip_stream(filepath, buffer))
     {
-        cerr << "failed to open zip stream: " << filepath << endl;
+        std::cerr << "failed to open zip stream: " << filepath << std::endl;
         return;
     }
 
@@ -746,7 +750,7 @@ void orcus_xlsx::read_rev_headers(const std::string& dir_path, const std::string
         get_config(), mp_impl->m_ns_repo, ooxml_tokens,
         reinterpret_cast<const char*>(&buffer[0]), buffer.size());
 
-    auto handler = std::make_unique<xml_simple_stream_handler>(
+    auto handler = std::make_unique<xml_stream_handler>(
         mp_impl->m_cxt, ooxml_tokens,
         std::make_unique<xlsx_revheaders_context>(mp_impl->m_cxt, ooxml_tokens));
 
@@ -759,17 +763,17 @@ void orcus_xlsx::read_rev_headers(const std::string& dir_path, const std::string
 
 void orcus_xlsx::read_rev_log(const std::string& dir_path, const std::string& file_name)
 {
-    string filepath = resolve_file_path(dir_path, file_name);
+    std::string filepath = resolve_file_path(dir_path, file_name);
     if (get_config().debug)
     {
-        cout << "---" << endl;
-        cout << "read_rev_log: file path = " << filepath << endl;
+        std::cout << "---" << std::endl;
+        std::cout << "read_rev_log: file path = " << filepath << std::endl;
     }
 
-    vector<unsigned char> buffer;
+    std::vector<unsigned char> buffer;
     if (!mp_impl->m_opc_reader.open_zip_stream(filepath, buffer))
     {
-        cerr << "failed to open zip stream: " << filepath << endl;
+        std::cerr << "failed to open zip stream: " << filepath << std::endl;
         return;
     }
 
@@ -780,7 +784,7 @@ void orcus_xlsx::read_rev_log(const std::string& dir_path, const std::string& fi
         get_config(), mp_impl->m_ns_repo, ooxml_tokens,
         reinterpret_cast<const char*>(&buffer[0]), buffer.size());
 
-    auto handler = std::make_unique<xml_simple_stream_handler>(
+    auto handler = std::make_unique<xml_stream_handler>(
         mp_impl->m_cxt, ooxml_tokens,
         std::make_unique<xlsx_revlog_context>(mp_impl->m_cxt, ooxml_tokens));
 
@@ -792,17 +796,17 @@ void orcus_xlsx::read_rev_log(const std::string& dir_path, const std::string& fi
 
 void orcus_xlsx::read_drawing(const std::string& dir_path, const std::string& file_name)
 {
-    string filepath = resolve_file_path(dir_path, file_name);
+    std::string filepath = resolve_file_path(dir_path, file_name);
     if (get_config().debug)
     {
-        cout << "---" << endl;
-        cout << "read_drawing: file path = " << filepath << endl;
+        std::cout << "---" << std::endl;
+        std::cout << "read_drawing: file path = " << filepath << std::endl;
     }
 
-    vector<unsigned char> buffer;
+    std::vector<unsigned char> buffer;
     if (!mp_impl->m_opc_reader.open_zip_stream(filepath, buffer))
     {
-        cerr << "failed to open zip stream: " << filepath << endl;
+        std::cerr << "failed to open zip stream: " << filepath << std::endl;
         return;
     }
 
