@@ -13,19 +13,17 @@
 
 #include "orcus/measurement.hpp"
 #include "orcus/string_pool.hpp"
-#include "orcus/global.hpp"
 
 #include <iostream>
-
-using namespace std;
+#include <limits>
 
 namespace orcus {
 
 namespace {
 
-class headers_attr_parser : public unary_function<xml_token_attr_t, void>
+class headers_attr_parser
 {
-    pstring m_guid;
+    std::string_view m_guid;
     long m_highest_revid;
     long m_version;
     bool m_disk_revisions;
@@ -33,7 +31,7 @@ class headers_attr_parser : public unary_function<xml_token_attr_t, void>
 public:
     headers_attr_parser() : m_highest_revid(-1), m_version(-1), m_disk_revisions(false) {}
 
-    pstring get_last_guid() const { return m_guid; }
+    std::string_view get_last_guid() const { return m_guid; }
     long get_highest_revid() const { return m_highest_revid; }
     long get_version() const { return m_version; }
     bool is_disk_revisions() const { return m_disk_revisions; }
@@ -66,13 +64,13 @@ public:
     }
 };
 
-class header_attr_parser : public unary_function<xml_token_attr_t, void>
+class header_attr_parser
 {
     string_pool* m_pool;
 
-    pstring m_guid;
-    pstring m_username;
-    pstring m_rid;
+    std::string_view m_guid;
+    std::string_view m_username;
+    std::string_view m_rid;
 
     date_time_t m_date_time;
     long m_next_sheet_id;
@@ -83,9 +81,9 @@ public:
     header_attr_parser(string_pool& pool) :
         m_pool(&pool), m_next_sheet_id(-1), m_min_revid(-1), m_max_revid(-1) {}
 
-    pstring get_guid() const { return m_guid; }
-    pstring get_username() const { return m_username; }
-    pstring get_rid() const { return m_rid; }
+    std::string_view get_guid() const { return m_guid; }
+    std::string_view get_username() const { return m_username; }
+    std::string_view get_rid() const { return m_rid; }
     date_time_t get_date_time() const { return m_date_time; }
     long get_next_sheet_id() const { return m_next_sheet_id; }
     long get_min_revid() const { return m_min_revid; }
@@ -101,7 +99,7 @@ public:
                     m_guid = attr.value;
                 break;
                 case XML_dateTime:
-                    m_date_time = to_date_time(attr.value);
+                    m_date_time = date_time_t::from_chars(attr.value);
                 break;
                 case XML_maxSheetId:
                     m_next_sheet_id = to_long(attr.value);
@@ -140,11 +138,6 @@ xlsx_revheaders_context::xlsx_revheaders_context(session_context& session_cxt, c
 
 xlsx_revheaders_context::~xlsx_revheaders_context() {}
 
-bool xlsx_revheaders_context::can_handle_element(xmlns_id_t /*ns*/, xml_token_t /*name*/) const
-{
-    return true;
-}
-
 xml_context_base* xlsx_revheaders_context::create_child_context(xmlns_id_t /*ns*/, xml_token_t /*name*/)
 {
     return nullptr;
@@ -154,7 +147,7 @@ void xlsx_revheaders_context::end_child_context(xmlns_id_t /*ns*/, xml_token_t /
 {
 }
 
-void xlsx_revheaders_context::start_element(xmlns_id_t ns, xml_token_t name, const vector<xml_token_attr_t>& attrs)
+void xlsx_revheaders_context::start_element(xmlns_id_t ns, xml_token_t name, const xml_token_attrs_t& attrs)
 {
     xml_token_pair_t parent = push_stack(ns, name);
     if (ns == NS_ooxml_xlsx)
@@ -166,29 +159,29 @@ void xlsx_revheaders_context::start_element(xmlns_id_t ns, xml_token_t name, con
                 xml_element_expected(parent, XMLNS_UNKNOWN_ID, XML_UNKNOWN_TOKEN);
                 headers_attr_parser func;
                 func = for_each(attrs.begin(), attrs.end(), func);
-                cout << "* last guid: " << func.get_last_guid() << endl;
-                cout << "* highest revision ID: " << func.get_highest_revid() << endl;
-                cout << "* version: " << func.get_version() << endl;
-                cout << "* disk revisions: " << func.is_disk_revisions() << endl;
+                std::cout << "* last guid: " << func.get_last_guid() << std::endl;
+                std::cout << "* highest revision ID: " << func.get_highest_revid() << std::endl;
+                std::cout << "* version: " << func.get_version() << std::endl;
+                std::cout << "* disk revisions: " << func.is_disk_revisions() << std::endl;
             }
             break;
             case XML_header:
             {
                 xml_element_expected(parent, NS_ooxml_xlsx, XML_headers);
-                header_attr_parser func(get_session_context().m_string_pool);
+                header_attr_parser func(get_session_context().spool);
                 func = for_each(attrs.begin(), attrs.end(), func);
-                cout << "* revision header (guid:" << func.get_guid() << ")" << endl;
-                cout << "  - timestamp: " << func.get_date_time().to_string() << endl;
-                cout << "  - user name: " << func.get_username() << endl;
+                std::cout << "* revision header (guid:" << func.get_guid() << ")" << std::endl;
+                std::cout << "  - timestamp: " << func.get_date_time().to_string() << std::endl;
+                std::cout << "  - user name: " << func.get_username() << std::endl;
 
                 if (func.get_min_revid() != -1 && func.get_max_revid() != -1)
-                    cout << "  - revision range: " << func.get_min_revid() << "-" << func.get_max_revid() << endl;
+                    std::cout << "  - revision range: " << func.get_min_revid() << "-" << func.get_max_revid() << std::endl;
 
                 long next_sheet = func.get_next_sheet_id();
                 if (next_sheet != -1)
-                    cout << "  - next available sheet: " << (next_sheet - 1) << endl;
+                    std::cout << "  - next available sheet: " << (next_sheet - 1) << std::endl;
 
-                cout << "  - revision log rid: " << func.get_rid() << endl;
+                std::cout << "  - revision log rid: " << func.get_rid() << std::endl;
                 // TODO : Intern the rid here when passing it to the revision log stream.
             }
             break;
@@ -196,17 +189,15 @@ void xlsx_revheaders_context::start_element(xmlns_id_t ns, xml_token_t name, con
             {
                 xml_element_expected(parent, NS_ooxml_xlsx, XML_header);
                 m_cur_sheet_ids.clear();
-                long n = single_long_attr_getter::get(attrs, NS_ooxml_xlsx, XML_count);
-                if (n > 0)
-                    m_cur_sheet_ids.reserve(n);
+                if (auto n = get_single_long_attr(attrs, NS_ooxml_xlsx, XML_count); n && *n > 0)
+                    m_cur_sheet_ids.reserve(*n);
             }
             break;
             case XML_sheetId:
             {
                 xml_element_expected(parent, NS_ooxml_xlsx, XML_sheetIdMap);
-                long val = single_long_attr_getter::get(attrs, NS_ooxml_xlsx, XML_val);
-                if (val > 0)
-                    m_cur_sheet_ids.push_back(val-1); // convert from 1-based to 0-based.
+                if (auto val = get_single_long_attr(attrs, NS_ooxml_xlsx, XML_val); val && *val > 0)
+                    m_cur_sheet_ids.push_back(*val - 1); // convert from 1-based to 0-based.
             }
             break;
             default:
@@ -223,10 +214,10 @@ bool xlsx_revheaders_context::end_element(xmlns_id_t ns, xml_token_t name)
         {
             case XML_sheetIdMap:
             {
-                cout << "  - sheet indices: ";
+                std::cout << "  - sheet indices: ";
                 for (size_t i = 0; i < m_cur_sheet_ids.size(); ++i)
-                    cout << m_cur_sheet_ids[i] << " ";
-                cout << endl;
+                    std::cout << m_cur_sheet_ids[i] << " ";
+                std::cout << std::endl;
             }
             break;
             default:
@@ -236,13 +227,13 @@ bool xlsx_revheaders_context::end_element(xmlns_id_t ns, xml_token_t name)
     return pop_stack(ns, name);
 }
 
-void xlsx_revheaders_context::characters(const pstring& /*str*/, bool /*transient*/)
+void xlsx_revheaders_context::characters(std::string_view /*str*/, bool /*transient*/)
 {
 }
 
 namespace {
 
-class rcc_attr_parser : public unary_function<xml_token_attr_t, void>
+class rcc_attr_parser
 {
     long m_revision_id;
     long m_sheet_id;
@@ -274,12 +265,12 @@ public:
     }
 };
 
-class rrc_attr_parser : public unary_function<xml_token_attr_t, void>
+class rrc_attr_parser
 {
     long m_revision_id;
     long m_sheet_id;
 
-    pstring m_ref;
+    std::string_view m_ref;
     xlsx_rev_row_column_action_t m_action_type;
 
     bool m_end_of_list;
@@ -291,7 +282,7 @@ public:
     long get_revision_id() const { return m_revision_id; }
     long get_sheet_id() const { return m_sheet_id; }
     bool is_end_of_list() const { return m_end_of_list; }
-    pstring get_ref() const { return m_ref; }
+    std::string_view get_ref() const { return m_ref; }
     xlsx_rev_row_column_action_t get_action_type() const { return m_action_type; }
 
     void operator() (const xml_token_attr_t& attr)
@@ -325,15 +316,15 @@ public:
     }
 };
 
-class cell_data_attr_parser : public unary_function<xml_token_attr_t, void>
+class cell_data_attr_parser
 {
-    pstring m_ref;
+    std::string_view m_ref;
     xlsx_cell_t m_type;
 
 public:
     cell_data_attr_parser() : m_type(xlsx_ct_numeric) {}
 
-    pstring get_ref() const { return m_ref; }
+    std::string_view get_ref() const { return m_ref; }
     xlsx_cell_t get_cell_type() const { return m_type; }
 
     void operator() (const xml_token_attr_t& attr)
@@ -366,11 +357,6 @@ xlsx_revlog_context::xlsx_revlog_context(session_context& session_cxt, const tok
 
 xlsx_revlog_context::~xlsx_revlog_context() {}
 
-bool xlsx_revlog_context::can_handle_element(xmlns_id_t /*ns*/, xml_token_t /*name*/) const
-{
-    return true;
-}
-
 xml_context_base* xlsx_revlog_context::create_child_context(xmlns_id_t /*ns*/, xml_token_t /*name*/)
 {
     return nullptr;
@@ -380,7 +366,7 @@ void xlsx_revlog_context::end_child_context(xmlns_id_t /*ns*/, xml_token_t /*nam
 {
 }
 
-void xlsx_revlog_context::start_element(xmlns_id_t ns, xml_token_t name, const vector<xml_token_attr_t>& attrs)
+void xlsx_revlog_context::start_element(xmlns_id_t ns, xml_token_t name, const xml_token_attrs_t& attrs)
 {
     xml_token_pair_t parent = push_stack(ns, name);
 
@@ -401,8 +387,8 @@ void xlsx_revlog_context::start_element(xmlns_id_t ns, xml_token_t name, const v
                 xml_element_expected(parent, NS_ooxml_xlsx, XML_revisions);
                 rcc_attr_parser func;
                 func = for_each(attrs.begin(), attrs.end(), func);
-                cout << "* revision id: " << func.get_revision_id() << "  type: cell change" << endl;
-                cout << "  - sheet index: " << func.get_sheet_id() << endl;
+                std::cout << "* revision id: " << func.get_revision_id() << "  type: cell change" << std::endl;
+                std::cout << "  - sheet index: " << func.get_sheet_id() << std::endl;
                 m_cur_cell_type = xlsx_ct_unknown;
             }
             break;
@@ -451,11 +437,11 @@ void xlsx_revlog_context::start_element(xmlns_id_t ns, xml_token_t name, const v
                 xml_element_expected(parent, NS_ooxml_xlsx, XML_revisions);
                 rrc_attr_parser func;
                 func = for_each(attrs.begin(), attrs.end(), func);
-                cout << "* revision id: " << func.get_revision_id() << "  type: row column insert delete" << endl;
-                cout << "  - sheet index: " << func.get_sheet_id() << endl;
-                cout << "  - action type: " << to_string(func.get_action_type()) << endl;
-                cout << "  - range: " << func.get_ref() << endl;
-                cout << "  - end of list: " << (func.is_end_of_list() ? "true":"false") << endl;
+                std::cout << "* revision id: " << func.get_revision_id() << "  type: row column insert delete" << std::endl;
+                std::cout << "  - sheet index: " << func.get_sheet_id() << std::endl;
+                std::cout << "  - action type: " << to_string(func.get_action_type()) << std::endl;
+                std::cout << "  - range: " << func.get_ref() << std::endl;
+                std::cout << "  - end of list: " << (func.is_end_of_list() ? "true":"false") << std::endl;
             }
             break;
             case XML_rsnm:
@@ -476,10 +462,10 @@ void xlsx_revlog_context::start_element(xmlns_id_t ns, xml_token_t name, const v
 
                 m_cur_formula = false;
                 m_cur_value = 0.0;
-                m_cur_string.clear();
+                m_cur_string = std::string_view{};
 
-                cout << "  - new cell position: " << func.get_ref() << endl;
-                cout << "  - new cell type: " << to_string(m_cur_cell_type) << endl;
+                std::cout << "  - new cell position: " << func.get_ref() << std::endl;
+                std::cout << "  - new cell type: " << to_string(m_cur_cell_type) << std::endl;
             }
             break;
             case XML_f:
@@ -528,28 +514,28 @@ bool xlsx_revlog_context::end_element(xmlns_id_t ns, xml_token_t name)
         {
             case XML_nc:
             {
-                cout << "  - new cell value: ";
+                std::cout << "  - new cell value: ";
                 switch (m_cur_cell_type)
                 {
                     case xlsx_ct_boolean:
                         if (m_cur_value != 0.0)
-                            cout << "true";
+                            std::cout << "true";
                         else
-                            cout << "false";
+                            std::cout << "false";
                     break;
                     case xlsx_ct_numeric:
                         if (m_cur_formula)
-                            cout << m_cur_string;
+                            std::cout << m_cur_string;
                         else
-                            cout << m_cur_value;
+                            std::cout << m_cur_value;
                     break;
                     case xlsx_ct_inline_string:
-                        cout << m_cur_string;
+                        std::cout << m_cur_string;
                     break;
                     default:
                         ;
                 }
-                cout << endl;
+                std::cout << std::endl;
             }
             break;
             default:
@@ -559,9 +545,9 @@ bool xlsx_revlog_context::end_element(xmlns_id_t ns, xml_token_t name)
     return pop_stack(ns, name);
 }
 
-void xlsx_revlog_context::characters(const pstring& str, bool transient)
+void xlsx_revlog_context::characters(std::string_view str, bool transient)
 {
-    const xml_token_pair_t& elem = get_current_element();
+    xml_token_pair_t elem = get_current_element();
     if (elem.first == NS_ooxml_xlsx)
     {
         switch (elem.second)
@@ -575,7 +561,7 @@ void xlsx_revlog_context::characters(const pstring& str, bool transient)
             case XML_t:
                 m_cur_string = str;
                 if (transient)
-                    m_cur_string = get_session_context().m_string_pool.intern(m_cur_string).first;
+                    m_cur_string = get_session_context().spool.intern(m_cur_string).first;
             break;
             default:
                 ;
