@@ -15,7 +15,6 @@
 #include "orcus/config.hpp"
 
 using namespace orcus;
-using namespace std;
 using namespace orcus::spreadsheet;
 using namespace orcus::spreadsheet::mock;
 
@@ -85,6 +84,15 @@ public:
     }
 };
 
+const std::string_view str_value =
+"Lorem ipsum dolor sit amet consectetur adipiscing elit. Quisque "
+"faucibus ex sapien vitae pellentesque sem placerat. In id cursus"
+" mi pretium tellus duis convallis. Tempus leo eu aenean sed diam"
+" urna tempor. Pulvinar vivamus fringilla lacus nec metus bibendu"
+"m egestas. Iaculis massa nisl malesuada lacinia integer nunc pos"
+"uere. Ut hendrerit semper vel class aptent taciti sociosqu. Ad l"
+"itora torquent per conubia nostra inceptos himenaeos.";
+
 class mock_sheet : public import_sheet
 {
     mock_array_formula m_array_formula;
@@ -104,6 +112,12 @@ public:
         assert(val == true);
     }
 
+    virtual void set_string(row_t row, col_t col, std::string_view s) override {
+        assert(row == -1);
+        assert(col == 0);
+        assert(s == str_value);
+    }
+
     virtual iface::import_array_formula* get_array_formula() override
     {
         return &m_array_formula;
@@ -113,15 +127,17 @@ public:
 class mock_sheet_properties : public import_sheet_properties
 {
 public:
-    void set_column_hidden(col_t col, bool hidden)
+    void set_column_hidden(col_t col, col_t col_span, bool hidden)
     {
         assert(col == 1);
+        assert(col_span == 1);
         assert(hidden);
     }
 
-    void set_row_hidden(row_t row, bool hidden)
+    void set_row_hidden(row_t row, row_t row_span, bool hidden)
     {
         assert(row == 3);
+        assert(row_span == 1);
         assert(hidden);
     }
 };
@@ -142,7 +158,7 @@ void test_cell_value()
 {
     mock_sheet sheet;
     mock_ref_resolver resolver;
-    session_context cxt(new xlsx_session_data);
+    session_context cxt(std::make_unique<xlsx_session_data>());
     config opt(format_t::xlsx);
     opt.structure_check = false;
 
@@ -151,11 +167,11 @@ void test_cell_value()
 
     orcus::xmlns_id_t ns = NS_ooxml_xlsx;
     orcus::xml_token_t elem = XML_c;
-    orcus::xml_attrs_t attrs;
+    orcus::xml_token_attrs_t attrs;
     context.start_element(ns, elem, attrs);
 
     {
-        xml_attrs_t val_attrs;
+        xml_token_attrs_t val_attrs;
         context.start_element(ns, XML_v, val_attrs);
         context.characters("5", false);
         context.end_element(ns, XML_v);
@@ -168,7 +184,7 @@ void test_cell_bool()
 {
     mock_sheet sheet;
     mock_ref_resolver resolver;
-    session_context cxt(new xlsx_session_data);
+    session_context cxt(std::make_unique<xlsx_session_data>());
     config opt(format_t::xlsx);
     opt.structure_check = false;
 
@@ -177,12 +193,12 @@ void test_cell_bool()
 
     orcus::xmlns_id_t ns = NS_ooxml_xlsx;
     orcus::xml_token_t elem = XML_c;
-    orcus::xml_attrs_t attrs;
+    orcus::xml_token_attrs_t attrs;
     attrs.push_back(xml_token_attr_t(NS_ooxml_xlsx, XML_t, "b", false));
     context.start_element(ns, elem, attrs);
 
     {
-        xml_attrs_t val_attrs;
+        xml_token_attrs_t val_attrs;
         context.start_element(ns, XML_v, val_attrs);
         context.characters("1", false);
         context.end_element(ns, XML_v);
@@ -195,7 +211,7 @@ void test_array_formula()
 {
     mock_sheet sheet;
     mock_ref_resolver resolver;
-    session_context cxt(new xlsx_session_data);
+    session_context cxt(std::make_unique<xlsx_session_data>());
     config opt(format_t::xlsx);
     opt.structure_check = false;
 
@@ -204,11 +220,11 @@ void test_array_formula()
 
     orcus::xmlns_id_t ns = NS_ooxml_xlsx;
     orcus::xml_token_t elem = XML_c;
-    orcus::xml_attrs_t attrs;
+    orcus::xml_token_attrs_t attrs;
     context.start_element(ns, elem, attrs);
 
     {
-        xml_attrs_t formula_attrs;
+        xml_token_attrs_t formula_attrs;
         formula_attrs.push_back(xml_token_attr_t(NS_ooxml_xlsx, XML_t, "array", false));
         formula_attrs.push_back(xml_token_attr_t(NS_ooxml_xlsx, XML_ref, "B3:B4", false));
         context.start_element(ns, XML_f, formula_attrs);
@@ -216,7 +232,7 @@ void test_array_formula()
         context.end_element(ns, XML_f);
     }
     {
-        xml_attrs_t val_attrs;
+        xml_token_attrs_t val_attrs;
         context.start_element(ns, XML_v, val_attrs);
         context.characters("5", false);
         context.end_element(ns, XML_v);
@@ -225,11 +241,40 @@ void test_array_formula()
     context.end_element(ns, elem);
 }
 
+void test_cell_string()
+{
+    mock_sheet sheet;
+    mock_ref_resolver resolver;
+    session_context cxt(std::make_unique<xlsx_session_data>());
+    config opt(format_t::xlsx);
+    opt.structure_check = false;
+
+    orcus::xlsx_sheet_context context(cxt, orcus::ooxml_tokens, 0, resolver, sheet);
+    context.set_config(opt);
+
+    orcus::xmlns_id_t ns = NS_ooxml_xlsx;
+
+    orcus::xml_token_attrs_t inline_attrs;
+    inline_attrs.push_back(xml_token_attr_t(ns, XML_t, "inlineStr", false));
+    context.start_element(ns, XML_c, inline_attrs);
+
+    {
+        xml_token_attrs_t val_attrs;
+        context.start_element(ns, XML_is, val_attrs);
+        context.start_element(ns, XML_t, val_attrs);
+        context.characters(str_value, false);
+        context.end_element(ns, XML_t);
+        context.end_element(ns, XML_is);
+    }
+
+    context.end_element(ns, XML_c);
+}
+
 void test_hidden_col()
 {
     mock_sheet2 sheet;
     mock_ref_resolver resolver;
-    session_context cxt(new xlsx_session_data);
+    session_context cxt(std::make_unique<xlsx_session_data>());
     config opt(format_t::xlsx);
     opt.structure_check = false;
 
@@ -238,7 +283,7 @@ void test_hidden_col()
 
     orcus::xmlns_id_t ns = NS_ooxml_xlsx;
     orcus::xml_token_t elem = XML_col;
-    orcus::xml_attrs_t attrs;
+    orcus::xml_token_attrs_t attrs;
     attrs.push_back(orcus::xml_token_attr_t(ns, XML_min, "2", false));
     attrs.push_back(orcus::xml_token_attr_t(ns, XML_max, "2", false));
     attrs.push_back(orcus::xml_token_attr_t(ns, XML_hidden, "1", false));
@@ -250,7 +295,7 @@ void test_hidden_row()
 {
     mock_sheet2 sheet;
     mock_ref_resolver resolver;
-    session_context cxt(new xlsx_session_data);
+    session_context cxt(std::make_unique<xlsx_session_data>());
     config opt(format_t::xlsx);
     opt.structure_check = false;
 
@@ -259,7 +304,7 @@ void test_hidden_row()
 
     orcus::xmlns_id_t ns = NS_ooxml_xlsx;
     orcus::xml_token_t elem = XML_row;
-    orcus::xml_attrs_t attrs;
+    orcus::xml_token_attrs_t attrs;
     attrs.push_back(orcus::xml_token_attr_t(ns, XML_r, "4", false));
     attrs.push_back(orcus::xml_token_attr_t(ns, XML_hidden, "1", false));
     context.start_element(ns, elem, attrs);
@@ -272,6 +317,7 @@ int main()
 {
     test_cell_value();
     test_cell_bool();
+    test_cell_string();
     test_array_formula();
     test_hidden_col();
     test_hidden_row();
