@@ -7,7 +7,6 @@
 
 #include <orcus/xml_structure_tree.hpp>
 #include <orcus/xml_namespace.hpp>
-#include <orcus/global.hpp>
 #include <orcus/stream.hpp>
 #include <orcus/parser_global.hpp>
 
@@ -16,11 +15,10 @@
 #include <iostream>
 #include <sstream>
 #include <vector>
-#include <boost/filesystem.hpp>
 
-using namespace std;
+#include "filesystem_env.hpp"
+
 using namespace orcus;
-namespace fs = boost::filesystem;
 
 const fs::path test_base_dir(SRCDIR"/test/xml-structure");
 
@@ -65,14 +63,14 @@ void test_basic()
     for (const fs::path& base_dir : base_dirs)
     {
         fs::path filepath = base_dir / "input.xml";
-        cout << filepath << endl;
+        std::cout << filepath << std::endl;
         auto lt = load_tree(xmlns_repo, filepath);
 
         std::ostringstream os;
         lt->tree.dump_compact(os);
-        string data_content = os.str();
-        cout << "--" << endl;
-        cout << data_content;
+        std::string data_content = os.str();
+        std::cout << "--" << std::endl;
+        std::cout << data_content;
 
         // Check the dump content against known datum.
         filepath = base_dir / "check.txt";
