@@ -16,7 +16,6 @@
 #include <cstdlib>
 #include <iostream>
 
-using namespace std;
 using namespace orcus;
 
 int main(int argc, char** argv)
@@ -25,8 +24,8 @@ int main(int argc, char** argv)
         return EXIT_FAILURE;
 
     string_pool sp;
-    spreadsheet::styles styles;
-    spreadsheet::import_styles istyles(styles, sp);
+    spreadsheet::styles styles_store;
+    spreadsheet::import_styles istyles(styles_store, sp);
 
     try
     {
