@@ -16,16 +16,21 @@
 
 using namespace orcus;
 
-int main(int argc, char** argv)
+int ORCUS_CLI_MAIN(int argc, arg_char_t** argv)
 {
     try
     {
-        spreadsheet::document doc;
+        bootstrap_program();
+
+        spreadsheet::range_size_t ss{1048576, 16384};
+        spreadsheet::document doc{ss};
         spreadsheet::view view(doc);
         spreadsheet::import_factory fact(doc, view);
         orcus_xlsx app(&fact);
 
-        if (parse_import_filter_args(argc, argv, fact, app, doc))
+        import_filter_arg_parser<arg_char_t> parser(fact, app, doc);
+
+        if (!parser.parse(argc, argv))
             return EXIT_FAILURE;
     }
     catch (const std::exception& e)
